Imports System.ComponentModel
Public Class PKVLeistungDataGridItem
    Implements IEditableObject

    Public Event PropertyChanged(ByVal sender As Object, ByVal e As PropertyChangedEventArgs)

    Private m_Parent As PKVLeistungDataGridListe

    Private m_Datum As DateTime = DateTime.MinValue
    Private WithEvents m_Leistung As PKVLeistung = Nothing
    Private m_Text As String = ""
    Private m_Punktzahl As Integer = 0
    Private m_Faktor As Decimal = CDec(0)
    Private m_Betrag As Decimal = CDec(0)
    ' Erzeugt ein neues Objekt aus existierenden Daten. Dieses Objekt
    ' wird nicht als neu gekennzeichnet.
    Public Sub New(ByVal parent As PKVLeistungDataGridListe, ByVal xDatum As DateTime, ByVal xLeistung As PKVLeistung, ByVal xText As String, ByVal xPunktzahl As Integer, ByVal xFaktor As Decimal, ByVal xBetrag As Decimal)
        m_Parent = parent
        m_Datum = xDatum
        m_Leistung = xLeistung
        m_Text = xText
        m_Punktzahl = xPunktzahl
        m_Faktor = xFaktor
        m_Betrag = xBetrag
        m_IsNew = False
    End Sub
    ' Erzeugt eine neue Instanz. Dieses Objekt wird als neu gekennzeichnet.
    Public Sub New(ByVal parent As PKVLeistungDataGridListe)
        m_Parent = parent
        m_Datum = m_Parent.AktuellesDatum
        m_Leistung = m_Parent.ZiffernListe.Nulleintrag
        m_Text = m_Leistung.Text
        m_Punktzahl = m_Leistung.Punktzahl
        If m_Leistung.MaxFaktor > m_Parent.MaxFactor Then
            m_Faktor = m_Parent.MaxFactor
        Else
            m_Faktor = m_Leistung.MaxFaktor
        End If
        m_Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
        m_IsNew = True
    End Sub



    Public Property Datum() As DateTime
        Get
            Return m_Datum
        End Get
        Set(ByVal Value As DateTime)
            m_Datum = Value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Datum"))
        End Set
    End Property
    Public Property Leistung() As PKVLeistung
        Get
            Return m_Leistung
        End Get
        Set(ByVal Value As PKVLeistung)
            m_Leistung = Value
            m_Text = m_Leistung.Text
            m_Punktzahl = m_Leistung.Punktzahl
            If m_Leistung.MaxFaktor > m_Parent.MaxFactor Then
                m_Faktor = m_Parent.MaxFactor
            Else
                m_Faktor = m_Leistung.MaxFaktor
            End If
            If m_Leistung.IstFestWert Then
                m_Betrag = m_Leistung.Festwert
            Else
                m_Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
            End If
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Leistung"))
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Text"))
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Punktzahl"))
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Faktor"))
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Betrag"))
        End Set
    End Property
    Public Property Text() As String
        Get
            Return m_Text
        End Get
        Set(ByVal Value As String)
            m_Text = Value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Text"))
        End Set
    End Property
    Public Property Punktzahl() As Integer
        Get
            Return m_Punktzahl
        End Get
        Set(ByVal Value As Integer)
            m_Punktzahl = Value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Punktzahl"))
            If Not m_Leistung.IstFestWert Then
                Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
            End If
        End Set
    End Property
    Public Property Faktor() As Decimal
        Get
            Return m_Faktor
        End Get
        Set(ByVal Value As Decimal)
            m_Faktor = Value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Faktor"))
            If Not m_Leistung.IstFestWert Then
                Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
            End If
        End Set
    End Property
    Public Property Betrag() As Decimal
        Get
            Return m_Betrag
        End Get
        Set(ByVal Value As Decimal)
            m_Betrag = Value
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Betrag"))
        End Set
    End Property
#Region " IEditableObject "
    Private m_Editing As Boolean
    Private m_IsNew As Boolean = True

    Private m_OldDatum As DateTime
    Private m_OldLeistung As PKVLeistung
    Private m_OldText As String
    Private m_OldPunktzahl As Integer
    Private m_OldFaktor As Decimal
    Private m_OldBetrag As Decimal


    Public Sub BeginEdit() Implements System.ComponentModel.IEditableObject.BeginEdit
        If Not m_Editing Then
            m_OldDatum = m_Datum
            m_OldLeistung = m_Leistung
            m_OldText = m_Text
            m_OldPunktzahl = m_Punktzahl
            m_OldFaktor = m_Faktor
            m_OldBetrag = m_Betrag
            m_Editing = True
        End If
    End Sub

    Public Sub CancelEdit() Implements System.ComponentModel.IEditableObject.CancelEdit
        If m_Editing Then
            If m_IsNew Then m_Parent.RemoveChild(Me)
            m_Datum = m_OldDatum
            m_Leistung = m_OldLeistung
            m_Text = m_OldText
            m_Punktzahl = m_OldPunktzahl
            m_Faktor = m_OldFaktor
            m_Betrag = m_OldBetrag
            m_Editing = False
        End If
    End Sub

    Public Sub EndEdit() Implements System.ComponentModel.IEditableObject.EndEdit
        m_Editing = False
        m_IsNew = False
    End Sub
#End Region

    Private Sub m_Leistung_Festwert_Geaendert() Handles m_Leistung.Festwert_Geaendert
        Punktzahl = m_Leistung.Punktzahl
        If m_Leistung.MaxFaktor > m_Parent.MaxFactor Then
            Faktor = m_Parent.MaxFactor
        Else
            Faktor = m_Leistung.MaxFaktor
        End If
        If m_Leistung.IstFestWert Then
            Betrag = m_Leistung.Festwert
        Else
            Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
        End If
    End Sub
    Private Sub m_Leistung_IstFestWert_Geaendert() Handles m_Leistung.IstFestWert_Geaendert
        Punktzahl = m_Leistung.Punktzahl
        If m_Leistung.MaxFaktor > m_Parent.MaxFactor Then
            Faktor = m_Parent.MaxFactor
        Else
            Faktor = m_Leistung.MaxFaktor
        End If
        If m_Leistung.IstFestWert Then
            Betrag = m_Leistung.Festwert
        Else
            Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
        End If
    End Sub
    Private Sub m_Leistung_MaxFaktor_Geaendert() Handles m_Leistung.MaxFaktor_Geaendert
        Punktzahl = m_Leistung.Punktzahl
        If m_Leistung.MaxFaktor > m_Parent.MaxFactor Then
            Faktor = m_Parent.MaxFactor
        Else
            Faktor = m_Leistung.MaxFaktor
        End If
        If m_Leistung.IstFestWert Then
            Betrag = m_Leistung.Festwert
        Else
            Betrag = Math.Round((CDec(m_Punktzahl) * m_Faktor * CDec(0.0582873)), 2)
        End If
    End Sub
    Private Sub m_Leistung_Punktzahl_Geaendert() Handles m_Leistung.Punktzahl_Geaendert
        Punktzahl = m_Leistung.Punktzahl
    End Sub
    Private Sub m_Leistung_Text_Geaendert() Handles m_Leistung.Text_Geaendert
        Text = m_Leistung.Text
    End Sub
    Private Sub m_Leistung_Ziffer_Geaendert() Handles m_Leistung.Ziffer_Geaendert
        RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs("Leistung"))
    End Sub
End Class
