Option Strict On
Imports System.ComponentModel
Imports System.Drawing
Namespace ValidatedControls
    <DesignerAttribute(GetType(DateTextBoxDesigner)), ToolboxBitmap(GetType(DateTextBox))> _
    Public Class DateTextBox
        Inherits ValidatedTextBox

        Private m_DateMinimum As DateTime = New DateTime(1800, 1, 1)
        Private m_DateMaximum As DateTime = New DateTime(2100, 1, 1)
        Private m_DateSpecialNullErlaubt As Boolean = False

        <Description("Minimum fr Datum."), Category("ValidatedTextBox")> _
        Property DateMinimum() As DateTime
            Get
                Return m_DateMinimum
            End Get
            Set(ByVal value As DateTime)
                m_DateMinimum = value
            End Set
        End Property
        <Description("Maximum fr Datum."), Category("ValidatedTextBox")> _
        Property DateMaximum() As DateTime
            Get
                Return m_DateMaximum
            End Get
            Set(ByVal value As DateTime)
                m_DateMaximum = value
            End Set
        End Property
        <Description("Legt fest, ob 00.00.0000 oder XX.00.0000 oder XX.XX.0000 als gltig akzeptiert werden."), Category("ValidatedTextBox")> _
        Property DateSpecialNullErlaubt() As Boolean
            Get
                Return m_DateSpecialNullErlaubt
            End Get
            Set(ByVal value As Boolean)
                m_DateSpecialNullErlaubt = value
            End Set
        End Property

        Public Sub New()
            'properties.Remove("TextMinLength")
            'properties.Remove("TextMaxLength")
            'properties.Remove("allowedChars")
            'properties.Remove("FilterChar")
            MyBase.TextMinLength = 6
            MyBase.TextMaxLength = 10
            MyBase.allowedChars = "1234567890."
            MyBase.FilterChar = "."c
        End Sub

        Friend Overrides Function ExtraFormatAndValueCheck(ByVal StringToCheck As String) As String
            'm_TextNullAllowed ist hier schon abgeklrt
            'StringToCheck kann also nicht "" sein.
            Dim Teil1, Teil2, Teil3 As Integer
            Teil1 = 0
            Teil2 = 0
            Teil3 = 0

            If StringToCheck.IndexOf(".") > -1 Then
                Dim Teile() As String = StringToCheck.Split("."c)
                If Teile.Length <> 3 Then
                    Return "Fehlerhafte Datumsangabe in " & m_ControlDescription & "."
                Else
                    For Each Txt As String In Teile
                        If Txt = "" Then
                            Return "Fehlerhafte Datumsangabe in " & m_ControlDescription & "."
                        End If
                    Next

                    Teil1 = CInt(Teile(0))
                    Teil2 = CInt(Teile(1))
                    Teil3 = CInt(Teile(2))
                End If
            Else
                Teil1 = CInt(StringToCheck.Substring(0, 2))
                Teil2 = CInt(StringToCheck.Substring(2, 2))
                Teil3 = CInt(StringToCheck.Substring(4))
            End If

            If Teil3 <> 0 Then
                If Teil3 < 100 Then
                    If Teil3 <= (DateTime.Now.Year - 2000) Then
                        Teil3 += 2000
                    Else
                        Teil3 += 1900
                    End If
                End If
            End If


            If m_DateSpecialNullErlaubt Then
                If Teil1 <> 0 Then
                    Try
                        Dim TestDatum As Date = New Date(Teil3, Teil2, Teil1)
                        If TestDatum > m_DateMaximum Then
                            Return "Datumsangabe in " & m_ControlDescription & " darf hchstens " & m_DateMaximum.ToString("dd.MM.yyyy") & " sein."
                        ElseIf TestDatum < m_DateMinimum Then
                            Return "Datumsangabe in " & m_ControlDescription & " muss mindestens " & m_DateMinimum.ToString("dd.MM.yyyy") & " sein."
                        End If
                    Catch ex As Exception
                        Return "Fehlerhafte Datumsangabe in " & m_ControlDescription & "."
                    End Try
                Else
                    If Teil2 <> 0 Then
                        Try
                            Dim TestDatum As Date = New Date(Teil3, Teil2, 1)
                            If TestDatum.Year > m_DateMaximum.Year Then
                                Return "Jahresangabe in " & m_ControlDescription & " darf hchstens " & m_DateMaximum.Year.ToString & " sein."
                            ElseIf TestDatum.Year = m_DateMaximum.Year Then
                                If TestDatum.Month > m_DateMaximum.Month Then
                                    Return "Monatsangabe fr das Jahr " & m_DateMaximum.Year.ToString & " in " & m_ControlDescription & " darf hchstens " & TextFunktionen.MonatsText(m_DateMaximum.Month) & " sein."
                                End If
                            ElseIf TestDatum.Year < m_DateMinimum.Year Then
                                Return "Jahresangabe in " & m_ControlDescription & " muss mindestens " & m_DateMinimum.Year.ToString & " sein."
                            ElseIf TestDatum.Year = m_DateMinimum.Year Then
                                If TestDatum.Month < m_DateMinimum.Month Then
                                    Return "Monatsangabe fr das Jahr " & m_DateMinimum.Year.ToString & " in " & m_ControlDescription & " muss mindestens " & TextFunktionen.MonatsText(m_DateMinimum.Month) & " sein."
                                End If
                            End If
                        Catch ex As Exception
                            Return "Fehlerhafte Datumsangabe in " & m_ControlDescription & "."
                        End Try
                    Else
                        If Teil3 <> 0 Then
                            'nur das Jahr ist nicht Null, alles andere ist Null
                            If Teil3 > m_DateMaximum.Year Then
                                Return "Jahresangabe in " & m_ControlDescription & " darf hchstens " & m_DateMaximum.Year.ToString & " sein."
                            ElseIf Teil3 < m_DateMinimum.Year Then
                                Return "Jahresangabe in " & m_ControlDescription & " muss mindestens " & m_DateMinimum.Year.ToString & " sein."
                            End If
                        End If
                    End If
                End If
            Else
                Try
                    Dim TestDatum As Date = New Date(Teil3, Teil2, Teil1)
                    If TestDatum > m_DateMaximum Then
                        Return "Datumsangabe in " & m_ControlDescription & " darf hchstens " & m_DateMaximum.ToString("dd.MM.yyyy") & " sein."
                    ElseIf TestDatum < m_DateMinimum Then
                        Return "Datumsangabe in " & m_ControlDescription & " muss mindestens " & m_DateMinimum.ToString("dd.MM.yyyy") & " sein."
                    End If
                Catch ex As Exception
                    Return "Fehlerhafte Datumsangabe in " & m_ControlDescription & "."
                End Try
            End If

            Return ""
        End Function

        Friend Overrides Function Formatieren(ByVal StringDaten As String) As String
            'Diese Prozedur wird nur aufgerufen, wenn FormatAndValueCheck keinen Fehler liefert
            '(ansonsten bleibt der Text in der TextBox unformatiert)
            'liefert in diesem Fall einen formatierten String
            'Die StringDaten knnen eine Zeichenfolge in formatierter oder unformatierter Form sein 
            If StringDaten = "" Then
                Return ""
            End If

            'Diese Funktion wird nur aufgerufen, wenn vorher mit ExtraFormatAndValueCheck geklrt 
            'wurde, dass formatierbare Daten vorliegen

            Dim Teil1, Teil2, Teil3 As Integer
            Teil1 = 0
            Teil2 = 0
            Teil3 = 0

            If StringDaten.IndexOf(".") > -1 Then
                Dim Teile() As String = StringDaten.Split("."c)
                Teil1 = CInt(Teile(0))
                Teil2 = CInt(Teile(1))
                Teil3 = CInt(Teile(2))
            Else
                Teil1 = CInt(StringDaten.Substring(0, 2))
                Teil2 = CInt(StringDaten.Substring(2, 2))
                Teil3 = CInt(StringDaten.Substring(4))
            End If

            If Teil3 <> 0 Then
                If Teil3 < 100 Then
                    If Teil3 <= (DateTime.Now.Year - 2000) Then
                        Teil3 += 2000
                    Else
                        Teil3 += 1900
                    End If
                End If
            End If


            Return Teil1.ToString.PadLeft(2, "0"c) & "." & Teil2.ToString.PadLeft(2, "0"c) & "." & Teil3.ToString.PadLeft(4, "0"c)

        End Function


    End Class

End Namespace
